/* A recursive-descent parser generated by peg 0.1.18 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#define YYRULECOUNT 55
#line 1 "heka_message_matcher_parser.leg"

/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set ts=2 et sw=2 tw=80: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/** @brief Hindsight message matcher parser @file */

#include <limits.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "heka_message_matcher_impl.h"
#include "luasandbox/util/heka_message.h"
#include "luasandbox/util/heka_message_matcher.h"

#ifndef _MSC_VER
#pragma GCC diagnostic ignored "-Wunused-parameter"
#pragma GCC diagnostic ignored "-Wunused-function"
#else
#pragma warning( disable : 4267 4244 )
#endif

typedef struct match_node_tmp {
  uint8_t id;
  uint8_t op;
  uint8_t val_mod;
  uint8_t val_type;
  uint8_t val_len;
  uint8_t var_len;
  uint8_t fi; // left node index for logical op
  uint8_t ai; // right node index for logical op
  char          *var;

  union {
    char    *s;
    double  d;
  } val;
} match_node_tmp;


typedef struct match_node_array {
  match_node_tmp *a;
  int            pos;
  int            size;
} match_node_array;


typedef struct input_string {
  const char *s;
  size_t     pos;
  size_t     size;
} input_string;


typedef struct context {
  match_node_array  out;
  match_node_array  ops;
  match_node_tmp    mn;
  struct tm         tms;
  int               cond_cnt;
  input_string      is;
} context;


#define YY_PARSE(T) static T
#define YY_CTX_LOCAL 1
#define YY_CTX_MEMBERS \
   context ctx;

#define YY_INPUT(yy, buf, result, max_size)                                    \
{                                                                              \
  input_string *is = &yy->ctx.is;                                              \
  result = is->size - is->pos;                                                 \
  if (result > 0) {                                                            \
    if (max_size >= result) {                                                  \
      memcpy(buf, &is->s[is->pos], result);                                    \
      is->pos += result;                                                       \
    } else {                                                                   \
      memcpy(buf, &is->s[is->pos], max_size);                                  \
      is->pos += max_size;                                                     \
      result = max_size;                                                       \
    }                                                                          \
  }                                                                            \
}


static void init_match_node(match_node_tmp *mn)
{
  memset(mn, 0, sizeof(match_node_tmp));
}


static void move_match_node(match_node_tmp *dest, match_node_tmp *src)
{
  memcpy(dest, src, sizeof(match_node_tmp));
  init_match_node(src); // dest now owns the memory, wipe the pointers
}


static void realloc_mna(match_node_array *mna)
{
  size_t bytes = sizeof(match_node_tmp) * ++mna->size;
  match_node_tmp *tmp = realloc(mna->a, bytes);
  if (tmp) {
    mna->a = tmp;
    init_match_node(&mna->a[mna->size - 1]);
  } else {
    fprintf(stderr, "realloc failed\n");
    exit(1);
  }
}


static void push_output(context *ctx, match_node_tmp *mn)
{
  if (!ctx->out.a || ctx->out.pos == ctx->out.size) {
    realloc_mna(&ctx->out);
  }
  move_match_node(&ctx->out.a[ctx->out.pos++], mn);
}


static void push_op(context *ctx, match_operation op)
{
  if (!ctx->ops.a) {
    realloc_mna(&ctx->ops);
    ctx->ops.a[ctx->ops.pos++].op = op;
    return;
  }

  if (op == OP_OPEN || op > ctx->ops.a[ctx->ops.pos - 1].op) {
    if (ctx->ops.pos == ctx->ops.size) {
      realloc_mna(&ctx->ops);
    }
    ctx->ops.a[ctx->ops.pos++].op = op;
  } else {
    push_output(ctx, &ctx->ops.a[ctx->ops.pos - 1]);
    ctx->ops.a[ctx->ops.pos - 1].op = op;
  }
}


static void pop_to_paren(context *ctx)
{
  for (; ctx->ops.pos > 0; --ctx->ops.pos) {
    match_node_tmp *op = &ctx->ops.a[ctx->ops.pos - 1];
    if (op->op == OP_OPEN) break;
    push_output(ctx, op);
  }
}


static void pop_all_ops(context *ctx)
{
  for (; ctx->ops.pos > 0; --ctx->ops.pos) {
    match_node_tmp *op = &ctx->ops.a[ctx->ops.pos - 1];
    if (op->op == OP_OPEN) continue;
    push_output(ctx, op);
  }
}


static void update_date(context *ctx, int year, int mon, int day)
{
  ctx->tms.tm_isdst = -1;
  ctx->tms.tm_year  = year - 1900;
  ctx->tms.tm_mon   = mon - 1;
  ctx->tms.tm_mday  = day;
}


static void update_time(context *ctx, int hour, int minute, int sec)
{
  ctx->tms.tm_hour  = hour;
  ctx->tms.tm_min   = minute;
  ctx->tms.tm_sec   = sec;
}


static void update_offset(context *ctx, char sign, int hour, int minute)
{
  ctx->mn.val.d += (hour * 3600 + minute * 60) * (sign == '-' ? -1 : 1);
}


static void set_field(context *ctx, char *name)
{
  ctx->mn.id = LSB_PB_FIELDS;
  ctx->mn.var_len = strlen(name);
  ctx->mn.var = malloc(ctx->mn.var_len + 1);
  if (!ctx->mn.var) {
    fprintf(stderr, "malloc failed\n");
    exit(1);
  }
  memcpy(ctx->mn.var, name, ctx->mn.var_len + 1);
}


static void set_timestamp(context *ctx)
{
  ctx->mn.id = LSB_PB_TIMESTAMP;
  ctx->mn.val_type = TYPE_NUMERIC;
  if (ctx->tms.tm_isdst == -1) {
    ctx->mn.val.d += mktime(&ctx->tms);
    ctx->mn.val.d *= 1e9;
  }
  memset(&ctx->tms, 0, sizeof(struct tm));
}


static void set_numeric_value(context *ctx, char *s)
{
  ctx->mn.val_type = TYPE_NUMERIC;
  ctx->mn.val.d = strtod(s, NULL);
}


static void set_string_value(context *ctx, char *s)
{
  ctx->mn.val_type = TYPE_STRING;
  int i, j;
  for (i = 0, j = 0; s[i]; ++i, ++j) {
    if (s[i] == '\\' && (s[i + 1] == '"' || s[i + 1] == '\'')) {
      ++i;
    }
    s[j] = s[i];
  }
  s[j] = 0;

  ctx->mn.val_len = j;
  ctx->mn.val.s = malloc(j + 1);
  if (!ctx->mn.val.s) {
    fprintf(stderr, "malloc failed\n");
    exit(1);
  }
  memcpy(ctx->mn.val.s, s, j + 1);
}


static void set_match_mod(context *ctx)
{
  if (ctx->mn.val_mod == PATTERN_MOD_NONE
     && strpbrk(ctx->mn.val.s, "^$*+?.[%-") == NULL) { // literal
    ctx->mn.val_mod = PATTERN_MOD_ESC;
  }
}


static bool check_string_len(char *s)
{
  int i, j;
  for (i = 0, j = 0; s[i]; ++i, ++j) {
    if (s[i] == '\\' && (s[i + 1] == '"' || s[i + 1] == '\'')) {
      ++i;
    }
  }
  return (j > UCHAR_MAX) ? false : true;
}


static int cond_cnt(context *ctx)
{
  return (++ctx->cond_cnt * 2 + 1 > UCHAR_MAX) ? 0 : 1;
}


#ifndef YY_MALLOC
#define YY_MALLOC(C, N)		malloc(N)
#endif
#ifndef YY_REALLOC
#define YY_REALLOC(C, P, N)	realloc(P, N)
#endif
#ifndef YY_FREE
#define YY_FREE(C, P)		free(P)
#endif
#ifndef YY_LOCAL
#define YY_LOCAL(T)	static T
#endif
#ifndef YY_ACTION
#define YY_ACTION(T)	static T
#endif
#ifndef YY_RULE
#define YY_RULE(T)	static T
#endif
#ifndef YY_PARSE
#define YY_PARSE(T)	T
#endif
#ifndef YYPARSE
#define YYPARSE		yyparse
#endif
#ifndef YYPARSEFROM
#define YYPARSEFROM	yyparsefrom
#endif
#ifndef YYRELEASE
#define YYRELEASE	yyrelease
#endif
#ifndef YY_BEGIN
#define YY_BEGIN	( yy->__begin= yy->__pos, 1)
#endif
#ifndef YY_END
#define YY_END		( yy->__end= yy->__pos, 1)
#endif
#ifdef YY_DEBUG
# define yyprintf(args)	fprintf args
#else
# define yyprintf(args)
#endif
#ifndef YYSTYPE
#define YYSTYPE	int
#endif
#ifndef YY_STACK_SIZE
#define YY_STACK_SIZE 128
#endif

#ifndef YY_BUFFER_SIZE
#define YY_BUFFER_SIZE 1024
#endif

#ifndef YY_PART

typedef struct _yycontext yycontext;
typedef void (*yyaction)(yycontext *yy, char *yytext, int yyleng);
typedef struct _yythunk { int begin, end;  yyaction  action;  struct _yythunk *next; } yythunk;

struct _yycontext {
  char     *__buf;
  int       __buflen;
  int       __pos;
  int       __limit;
  char     *__text;
  int       __textlen;
  int       __begin;
  int       __end;
  int       __textmax;
  yythunk  *__thunks;
  int       __thunkslen;
  int       __thunkpos;
  YYSTYPE   __;
  YYSTYPE  *__val;
  YYSTYPE  *__vals;
  int       __valslen;
#ifdef YY_CTX_MEMBERS
  YY_CTX_MEMBERS
#endif
};

#ifdef YY_CTX_LOCAL
#define YY_CTX_PARAM_	yycontext *yyctx,
#define YY_CTX_PARAM	yycontext *yyctx
#define YY_CTX_ARG_	yyctx,
#define YY_CTX_ARG	yyctx
#ifndef YY_INPUT
#define YY_INPUT(yy, buf, result, max_size)		\
  {							\
    int yyc= getchar();					\
    result= (EOF == yyc) ? 0 : (*(buf)= yyc, 1);	\
    yyprintf((stderr, "<%c>", yyc));			\
  }
#endif
#else
#define YY_CTX_PARAM_
#define YY_CTX_PARAM
#define YY_CTX_ARG_
#define YY_CTX_ARG
yycontext _yyctx= { 0, 0 };
yycontext *yyctx= &_yyctx;
#ifndef YY_INPUT
#define YY_INPUT(buf, result, max_size)			\
  {							\
    int yyc= getchar();					\
    result= (EOF == yyc) ? 0 : (*(buf)= yyc, 1);	\
    yyprintf((stderr, "<%c>", yyc));			\
  }
#endif
#endif

YY_LOCAL(int) yyrefill(yycontext *yy)
{
  int yyn;
  while (yy->__buflen - yy->__pos < 512)
    {
      yy->__buflen *= 2;
      yy->__buf= (char *)YY_REALLOC(yy, yy->__buf, yy->__buflen);
    }
#ifdef YY_CTX_LOCAL
  YY_INPUT(yy, (yy->__buf + yy->__pos), yyn, (yy->__buflen - yy->__pos));
#else
  YY_INPUT((yy->__buf + yy->__pos), yyn, (yy->__buflen - yy->__pos));
#endif
  if (!yyn) return 0;
  yy->__limit += yyn;
  return 1;
}

YY_LOCAL(int) yymatchDot(yycontext *yy)
{
  if (yy->__pos >= yy->__limit && !yyrefill(yy)) return 0;
  ++yy->__pos;
  return 1;
}

YY_LOCAL(int) yymatchChar(yycontext *yy, int c)
{
  if (yy->__pos >= yy->__limit && !yyrefill(yy)) return 0;
  if ((unsigned char)yy->__buf[yy->__pos] == c)
    {
      ++yy->__pos;
      yyprintf((stderr, "  ok   yymatchChar(yy, %c) @ %s\n", c, yy->__buf+yy->__pos));
      return 1;
    }
  yyprintf((stderr, "  fail yymatchChar(yy, %c) @ %s\n", c, yy->__buf+yy->__pos));
  return 0;
}

YY_LOCAL(int) yymatchString(yycontext *yy, const char *s)
{
  int yysav= yy->__pos;
  while (*s)
    {
      if (yy->__pos >= yy->__limit && !yyrefill(yy)) return 0;
      if (yy->__buf[yy->__pos] != *s)
        {
          yy->__pos= yysav;
          return 0;
        }
      ++s;
      ++yy->__pos;
    }
  return 1;
}

YY_LOCAL(int) yymatchClass(yycontext *yy, unsigned char *bits)
{
  int c;
  if (yy->__pos >= yy->__limit && !yyrefill(yy)) return 0;
  c= (unsigned char)yy->__buf[yy->__pos];
  if (bits[c >> 3] & (1 << (c & 7)))
    {
      ++yy->__pos;
      yyprintf((stderr, "  ok   yymatchClass @ %s\n", yy->__buf+yy->__pos));
      return 1;
    }
  yyprintf((stderr, "  fail yymatchClass @ %s\n", yy->__buf+yy->__pos));
  return 0;
}

YY_LOCAL(void) yyDo(yycontext *yy, yyaction action, int begin, int end)
{
  while (yy->__thunkpos >= yy->__thunkslen)
    {
      yy->__thunkslen *= 2;
      yy->__thunks= (yythunk *)YY_REALLOC(yy, yy->__thunks, sizeof(yythunk) * yy->__thunkslen);
    }
  yy->__thunks[yy->__thunkpos].begin=  begin;
  yy->__thunks[yy->__thunkpos].end=    end;
  yy->__thunks[yy->__thunkpos].action= action;
  ++yy->__thunkpos;
}

YY_LOCAL(int) yyText(yycontext *yy, int begin, int end)
{
  int yyleng= end - begin;
  if (yyleng <= 0)
    yyleng= 0;
  else
    {
      while (yy->__textlen < (yyleng + 1))
	{
	  yy->__textlen *= 2;
	  yy->__text= (char *)YY_REALLOC(yy, yy->__text, yy->__textlen);
	}
      memcpy(yy->__text, yy->__buf + begin, yyleng);
    }
  yy->__text[yyleng]= '\0';
  return yyleng;
}

YY_LOCAL(void) yyDone(yycontext *yy)
{
  int pos;
  for (pos= 0;  pos < yy->__thunkpos;  ++pos)
    {
      yythunk *thunk= &yy->__thunks[pos];
      int yyleng= thunk->end ? yyText(yy, thunk->begin, thunk->end) : thunk->begin;
      yyprintf((stderr, "DO [%d] %p %s\n", pos, thunk->action, yy->__text));
      thunk->action(yy, yy->__text, yyleng);
    }
  yy->__thunkpos= 0;
}

YY_LOCAL(void) yyCommit(yycontext *yy)
{
  if ((yy->__limit -= yy->__pos))
    {
      memmove(yy->__buf, yy->__buf + yy->__pos, yy->__limit);
    }
  yy->__begin -= yy->__pos;
  yy->__end -= yy->__pos;
  yy->__pos= yy->__thunkpos= 0;
}

YY_LOCAL(int) yyAccept(yycontext *yy, int tp0)
{
  if (tp0)
    {
      fprintf(stderr, "accept denied at %d\n", tp0);
      return 0;
    }
  else
    {
      yyDone(yy);
      yyCommit(yy);
    }
  return 1;
}

YY_LOCAL(void) yyPush(yycontext *yy, char *text, int count)
{
  yy->__val += count;
  while (yy->__valslen <= yy->__val - yy->__vals)
    {
      long offset= yy->__val - yy->__vals;
      int olen = yy->__valslen;
      yy->__valslen *= 2;
      yy->__vals= (YYSTYPE *)YY_REALLOC(yy, yy->__vals, sizeof(YYSTYPE) * yy->__valslen);
      yy->__val= yy->__vals + offset;
      memset(yy->__vals + olen, 0, sizeof(YYSTYPE) * olen);
    }
}
YY_LOCAL(void) yyPop(yycontext *yy, char *text, int count)   { yy->__val -= count; }
YY_LOCAL(void) yySet(yycontext *yy, char *text, int count)   { yy->__val[count]= yy->__; }

#endif /* YY_PART */

#define	YYACCEPT	yyAccept(yy, yythunkpos0)

YY_RULE(int) yy_second_frac(yycontext *yy); /* 55 */
YY_RULE(int) yy_second(yycontext *yy); /* 54 */
YY_RULE(int) yy_minute(yycontext *yy); /* 53 */
YY_RULE(int) yy_hour(yycontext *yy); /* 52 */
YY_RULE(int) yy_timeoffset(yycontext *yy); /* 51 */
YY_RULE(int) yy_partialtime(yycontext *yy); /* 50 */
YY_RULE(int) yy_fulltime(yycontext *yy); /* 49 */
YY_RULE(int) yy_day(yycontext *yy); /* 48 */
YY_RULE(int) yy_month(yycontext *yy); /* 47 */
YY_RULE(int) yy_year(yycontext *yy); /* 46 */
YY_RULE(int) yy_fulldate(yycontext *yy); /* 45 */
YY_RULE(int) yy_rfc3339(yycontext *yy); /* 44 */
YY_RULE(int) yy_ts_quoted(yycontext *yy); /* 43 */
YY_RULE(int) yy_zero_to_255(yycontext *yy); /* 42 */
YY_RULE(int) yy_index(yycontext *yy); /* 41 */
YY_RULE(int) yy_nil(yycontext *yy); /* 40 */
YY_RULE(int) yy_fields(yycontext *yy); /* 39 */
YY_RULE(int) yy_exponent(yycontext *yy); /* 38 */
YY_RULE(int) yy_decimal(yycontext *yy); /* 37 */
YY_RULE(int) yy_number(yycontext *yy); /* 36 */
YY_RULE(int) yy_sign(yycontext *yy); /* 35 */
YY_RULE(int) yy_numeric_value(yycontext *yy); /* 34 */
YY_RULE(int) yy_numeric_headers(yycontext *yy); /* 33 */
YY_RULE(int) yy_string_match_mod(yycontext *yy); /* 32 */
YY_RULE(int) yy_string_match(yycontext *yy); /* 31 */
YY_RULE(int) yy_string_value(yycontext *yy); /* 30 */
YY_RULE(int) yy_string_headers(yycontext *yy); /* 29 */
YY_RULE(int) yy_boolean(yycontext *yy); /* 28 */
YY_RULE(int) yy_false(yycontext *yy); /* 27 */
YY_RULE(int) yy_true(yycontext *yy); /* 26 */
YY_RULE(int) yy_relational(yycontext *yy); /* 25 */
YY_RULE(int) yy_op_lt(yycontext *yy); /* 24 */
YY_RULE(int) yy_op_lte(yycontext *yy); /* 23 */
YY_RULE(int) yy_op_gt(yycontext *yy); /* 22 */
YY_RULE(int) yy_op_gte(yycontext *yy); /* 21 */
YY_RULE(int) yy_op_sne(yycontext *yy); /* 20 */
YY_RULE(int) yy_op_seq(yycontext *yy); /* 19 */
YY_RULE(int) yy_op_ne(yycontext *yy); /* 18 */
YY_RULE(int) yy_op_eq(yycontext *yy); /* 17 */
YY_RULE(int) yy_boolean_test(yycontext *yy); /* 16 */
YY_RULE(int) yy_ts_test(yycontext *yy); /* 15 */
YY_RULE(int) yy_field_test(yycontext *yy); /* 14 */
YY_RULE(int) yy_numeric_test(yycontext *yy); /* 13 */
YY_RULE(int) yy_string_test(yycontext *yy); /* 12 */
YY_RULE(int) yy_close(yycontext *yy); /* 11 */
YY_RULE(int) yy_open(yycontext *yy); /* 10 */
YY_RULE(int) yy_test(yycontext *yy); /* 9 */
YY_RULE(int) yy_and(yycontext *yy); /* 8 */
YY_RULE(int) yy_expr(yycontext *yy); /* 7 */
YY_RULE(int) yy_or(yycontext *yy); /* 6 */
YY_RULE(int) yy_anded(yycontext *yy); /* 5 */
YY_RULE(int) yy_eol(yycontext *yy); /* 4 */
YY_RULE(int) yy_ored(yycontext *yy); /* 3 */
YY_RULE(int) yy_sp(yycontext *yy); /* 2 */
YY_RULE(int) yy_match(yycontext *yy); /* 1 */

YY_ACTION(void) yy_1_nil(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_nil\n"));
  {
#line 376
  yy->ctx.mn.val_type = TYPE_NIL;;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_second_frac(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_second_frac\n"));
  {
#line 374
  yy->ctx.mn.val.d += strtod(yytext, NULL);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_second(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_second\n"));
  {
#line 373
  __ = atoi(yytext);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_minute(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_minute\n"));
  {
#line 369
  __ = atoi(yytext);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_hour(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_hour\n"));
  {
#line 368
  __ = atoi(yytext);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_2_timeoffset(yycontext *yy, char *yytext, int yyleng)
{
#define m yy->__val[-1]
#define h yy->__val[-2]
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_2_timeoffset\n"));
  {
#line 364
  update_offset(&yy->ctx, yytext[0], h, m);
  }
#undef yythunkpos
#undef yypos
#undef yy
#undef m
#undef h
}
YY_ACTION(void) yy_1_timeoffset(yycontext *yy, char *yytext, int yyleng)
{
#define m yy->__val[-1]
#define h yy->__val[-2]
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_timeoffset\n"));
  {
#line 363
  update_offset(&yy->ctx, '+', 0, 0);
  }
#undef yythunkpos
#undef yypos
#undef yy
#undef m
#undef h
}
YY_ACTION(void) yy_1_partialtime(yycontext *yy, char *yytext, int yyleng)
{
#define s yy->__val[-1]
#define m yy->__val[-2]
#define h yy->__val[-3]
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_partialtime\n"));
  {
#line 362
  update_time(&yy->ctx, h, m, s);
  }
#undef yythunkpos
#undef yypos
#undef yy
#undef s
#undef m
#undef h
}
YY_ACTION(void) yy_1_day(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_day\n"));
  {
#line 358
  __ = atoi(yytext);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_month(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_month\n"));
  {
#line 353
  __ = atoi(yytext);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_year(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_year\n"));
  {
#line 350
  __ = atoi(yytext);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_fulldate(yycontext *yy, char *yytext, int yyleng)
{
#define d yy->__val[-1]
#define m yy->__val[-2]
#define y yy->__val[-3]
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_fulldate\n"));
  {
#line 349
  update_date(&yy->ctx, y, m, d);
  }
#undef yythunkpos
#undef yypos
#undef yy
#undef d
#undef m
#undef y
}
YY_ACTION(void) yy_1_ts_test(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_ts_test\n"));
  {
#line 347
  set_timestamp(&yy->ctx);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_index(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_index\n"));
  {
#line 341
  __ = atoi(yytext);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_3_fields(yycontext *yy, char *yytext, int yyleng)
{
#define a yy->__val[-1]
#define f yy->__val[-2]
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_3_fields\n"));
  {
#line 340
  yy->ctx.mn.ai = a;
  }
#undef yythunkpos
#undef yypos
#undef yy
#undef a
#undef f
}
YY_ACTION(void) yy_2_fields(yycontext *yy, char *yytext, int yyleng)
{
#define a yy->__val[-1]
#define f yy->__val[-2]
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_2_fields\n"));
  {
#line 340
  yy->ctx.mn.fi = f;
  }
#undef yythunkpos
#undef yypos
#undef yy
#undef a
#undef f
}
YY_ACTION(void) yy_1_fields(yycontext *yy, char *yytext, int yyleng)
{
#define a yy->__val[-1]
#define f yy->__val[-2]
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_fields\n"));
  {
#line 340
  set_field(&yy->ctx, yytext);
  }
#undef yythunkpos
#undef yypos
#undef yy
#undef a
#undef f
}
YY_ACTION(void) yy_1_numeric_value(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_numeric_value\n"));
  {
#line 330
  set_numeric_value(&yy->ctx, yytext);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_2_numeric_headers(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_2_numeric_headers\n"));
  {
#line 327
  yy->ctx.mn.id = LSB_PB_PID;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_numeric_headers(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_numeric_headers\n"));
  {
#line 326
  yy->ctx.mn.id = LSB_PB_SEVERITY;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_string_match_mod(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_string_match_mod\n"));
  {
#line 324
  yy->ctx.mn.val_mod = PATTERN_MOD_ESC;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_string_match(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_string_match\n"));
  {
#line 322
  set_match_mod(&yy->ctx);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_string_value(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_string_value\n"));
  {
#line 320
  set_string_value(&yy->ctx, yytext);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_6_string_headers(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_6_string_headers\n"));
  {
#line 316
  yy->ctx.mn.id = LSB_PB_UUID;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_5_string_headers(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_5_string_headers\n"));
  {
#line 315
  yy->ctx.mn.id = LSB_PB_PAYLOAD;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_4_string_headers(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_4_string_headers\n"));
  {
#line 314
  yy->ctx.mn.id = LSB_PB_ENV_VERSION;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_3_string_headers(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_3_string_headers\n"));
  {
#line 313
  yy->ctx.mn.id = LSB_PB_HOSTNAME;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_2_string_headers(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_2_string_headers\n"));
  {
#line 312
  yy->ctx.mn.id = LSB_PB_LOGGER;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_string_headers(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_string_headers\n"));
  {
#line 311
  yy->ctx.mn.id = LSB_PB_TYPE;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_close(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_close\n"));
  {
#line 307
  pop_to_paren(&yy->ctx);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_open(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_open\n"));
  {
#line 306
  push_op(&yy->ctx, OP_OPEN);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_or(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_or\n"));
  {
#line 305
  push_op(&yy->ctx, OP_OR);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_and(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_and\n"));
  {
#line 304
  push_op(&yy->ctx, OP_AND);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_2_boolean(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_2_boolean\n"));
  {
#line 302
  yy->ctx.mn.val_type = TYPE_FALSE;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_boolean(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_boolean\n"));
  {
#line 301
  yy->ctx.mn.val_type = TYPE_TRUE;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_2_boolean_test(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_2_boolean_test\n"));
  {
#line 300
  yy->ctx.mn.op = OP_FALSE;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_boolean_test(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_boolean_test\n"));
  {
#line 299
  yy->ctx.mn.op = OP_TRUE;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_op_lt(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_op_lt\n"));
  {
#line 290
  yy->ctx.mn.op = OP_LT;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_op_lte(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_op_lte\n"));
  {
#line 289
  yy->ctx.mn.op = OP_LTE;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_op_gt(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_op_gt\n"));
  {
#line 288
  yy->ctx.mn.op = OP_GT;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_op_gte(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_op_gte\n"));
  {
#line 287
  yy->ctx.mn.op = OP_GTE;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_op_sne(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_op_sne\n"));
  {
#line 286
  yy->ctx.mn.op = OP_NRE;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_op_seq(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_op_seq\n"));
  {
#line 285
  yy->ctx.mn.op = OP_RE;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_op_ne(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_op_ne\n"));
  {
#line 284
  yy->ctx.mn.op = OP_NE;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_op_eq(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_op_eq\n"));
  {
#line 283
  yy->ctx.mn.op = OP_EQ;
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_test(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_test\n"));
  {
#line 281
  push_output(&yy->ctx, &yy->ctx.mn);
  }
#undef yythunkpos
#undef yypos
#undef yy
}
YY_ACTION(void) yy_1_match(yycontext *yy, char *yytext, int yyleng)
{
#define __ yy->__
#define yypos yy->__pos
#define yythunkpos yy->__thunkpos
  yyprintf((stderr, "do yy_1_match\n"));
  {
#line 272
  pop_all_ops(&yy->ctx);
  }
#undef yythunkpos
#undef yypos
#undef yy
}

YY_RULE(int) yy_second_frac(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "second_frac"));  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l1;
#undef yytext
#undef yyleng
  }  if (!yy_decimal(yy)) goto l1;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l1;
#undef yytext
#undef yyleng
  }  yyDo(yy, yy_1_second_frac, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "second_frac", yy->__buf+yy->__pos));
  return 1;
  l1:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "second_frac", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_second(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "second"));
  {  int yypos3= yy->__pos, yythunkpos3= yy->__thunkpos;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l4;
#undef yytext
#undef yyleng
  }  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\077\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l4;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l4;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l4;
#undef yytext
#undef yyleng
  }  goto l3;
  l4:;	  yy->__pos= yypos3; yy->__thunkpos= yythunkpos3;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l2;
#undef yytext
#undef yyleng
  }  if (!yymatchString(yy, "60")) goto l2;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l2;
#undef yytext
#undef yyleng
  }
  }
  l3:;	  yyDo(yy, yy_1_second, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "second", yy->__buf+yy->__pos));
  return 1;
  l2:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "second", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_minute(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "minute"));  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l5;
#undef yytext
#undef yyleng
  }  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\077\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l5;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l5;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l5;
#undef yytext
#undef yyleng
  }  yyDo(yy, yy_1_minute, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "minute", yy->__buf+yy->__pos));
  return 1;
  l5:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "minute", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_hour(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "hour"));
  {  int yypos7= yy->__pos, yythunkpos7= yy->__thunkpos;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l8;
#undef yytext
#undef yyleng
  }  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l8;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l8;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l8;
#undef yytext
#undef yyleng
  }  goto l7;
  l8:;	  yy->__pos= yypos7; yy->__thunkpos= yythunkpos7;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l6;
#undef yytext
#undef yyleng
  }  if (!yymatchChar(yy, '2')) goto l6;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\017\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l6;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l6;
#undef yytext
#undef yyleng
  }
  }
  l7:;	  yyDo(yy, yy_1_hour, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "hour", yy->__buf+yy->__pos));
  return 1;
  l6:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "hour", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_timeoffset(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;  yyDo(yy, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "timeoffset"));
  {  int yypos10= yy->__pos, yythunkpos10= yy->__thunkpos;  if (!yymatchChar(yy, 'Z')) goto l11;  yyDo(yy, yy_1_timeoffset, yy->__begin, yy->__end);  goto l10;
  l11:;	  yy->__pos= yypos10; yy->__thunkpos= yythunkpos10;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l9;
#undef yytext
#undef yyleng
  }  if (!yy_sign(yy)) goto l9;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l9;
#undef yytext
#undef yyleng
  }  if (!yy_hour(yy)) goto l9;  yyDo(yy, yySet, -2, 0);  if (!yy_minute(yy)) goto l9;  yyDo(yy, yySet, -1, 0);  yyDo(yy, yy_2_timeoffset, yy->__begin, yy->__end);
  }
  l10:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "timeoffset", yy->__buf+yy->__pos));  yyDo(yy, yyPop, 2, 0);
  return 1;
  l9:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "timeoffset", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_partialtime(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;  yyDo(yy, yyPush, 3, 0);
  yyprintf((stderr, "%s\n", "partialtime"));  if (!yy_hour(yy)) goto l12;  yyDo(yy, yySet, -3, 0);  if (!yymatchChar(yy, ':')) goto l12;  if (!yy_minute(yy)) goto l12;  yyDo(yy, yySet, -2, 0);  if (!yymatchChar(yy, ':')) goto l12;  if (!yy_second(yy)) goto l12;  yyDo(yy, yySet, -1, 0);
  {  int yypos13= yy->__pos, yythunkpos13= yy->__thunkpos;  if (!yy_second_frac(yy)) goto l13;  goto l14;
  l13:;	  yy->__pos= yypos13; yy->__thunkpos= yythunkpos13;
  }
  l14:;	  yyDo(yy, yy_1_partialtime, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "partialtime", yy->__buf+yy->__pos));  yyDo(yy, yyPop, 3, 0);
  return 1;
  l12:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "partialtime", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_fulltime(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "fulltime"));  if (!yy_partialtime(yy)) goto l15;  if (!yy_timeoffset(yy)) goto l15;
  yyprintf((stderr, "  ok   %s @ %s\n", "fulltime", yy->__buf+yy->__pos));
  return 1;
  l15:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "fulltime", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_day(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "day"));
  {  int yypos17= yy->__pos, yythunkpos17= yy->__thunkpos;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l18;
#undef yytext
#undef yyleng
  }  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\006\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l18;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l18;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l18;
#undef yytext
#undef yyleng
  }  goto l17;
  l18:;	  yy->__pos= yypos17; yy->__thunkpos= yythunkpos17;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l19;
#undef yytext
#undef yyleng
  }  if (!yymatchChar(yy, '0')) goto l19;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\376\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l19;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l19;
#undef yytext
#undef yyleng
  }  goto l17;
  l19:;	  yy->__pos= yypos17; yy->__thunkpos= yythunkpos17;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l16;
#undef yytext
#undef yyleng
  }  if (!yymatchChar(yy, '3')) goto l16;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l16;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l16;
#undef yytext
#undef yyleng
  }
  }
  l17:;	  yyDo(yy, yy_1_day, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "day", yy->__buf+yy->__pos));
  return 1;
  l16:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "day", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_month(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "month"));
  {  int yypos21= yy->__pos, yythunkpos21= yy->__thunkpos;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l22;
#undef yytext
#undef yyleng
  }  if (!yymatchChar(yy, '0')) goto l22;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\376\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l22;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l22;
#undef yytext
#undef yyleng
  }  goto l21;
  l22:;	  yy->__pos= yypos21; yy->__thunkpos= yythunkpos21;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l20;
#undef yytext
#undef yyleng
  }  if (!yymatchChar(yy, '1')) goto l20;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l20;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l20;
#undef yytext
#undef yyleng
  }
  }
  l21:;	  yyDo(yy, yy_1_month, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "month", yy->__buf+yy->__pos));
  return 1;
  l20:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "month", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_year(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "year"));  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l23;
#undef yytext
#undef yyleng
  }  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l23;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l23;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l23;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l23;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l23;
#undef yytext
#undef yyleng
  }  yyDo(yy, yy_1_year, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "year", yy->__buf+yy->__pos));
  return 1;
  l23:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "year", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_fulldate(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;  yyDo(yy, yyPush, 3, 0);
  yyprintf((stderr, "%s\n", "fulldate"));  if (!yy_year(yy)) goto l24;  yyDo(yy, yySet, -3, 0);  if (!yymatchChar(yy, '-')) goto l24;  if (!yy_month(yy)) goto l24;  yyDo(yy, yySet, -2, 0);  if (!yymatchChar(yy, '-')) goto l24;  if (!yy_day(yy)) goto l24;  yyDo(yy, yySet, -1, 0);  yyDo(yy, yy_1_fulldate, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "fulldate", yy->__buf+yy->__pos));  yyDo(yy, yyPop, 3, 0);
  return 1;
  l24:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "fulldate", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_rfc3339(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "rfc3339"));  if (!yy_fulldate(yy)) goto l25;  if (!yymatchChar(yy, 'T')) goto l25;  if (!yy_fulltime(yy)) goto l25;
  yyprintf((stderr, "  ok   %s @ %s\n", "rfc3339", yy->__buf+yy->__pos));
  return 1;
  l25:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "rfc3339", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_ts_quoted(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "ts_quoted"));
  {  int yypos27= yy->__pos, yythunkpos27= yy->__thunkpos;  if (!yymatchChar(yy, '"')) goto l28;  if (!yy_rfc3339(yy)) goto l28;  if (!yymatchChar(yy, '"')) goto l28;  goto l27;
  l28:;	  yy->__pos= yypos27; yy->__thunkpos= yythunkpos27;  if (!yymatchChar(yy, '\'')) goto l26;  if (!yy_rfc3339(yy)) goto l26;  if (!yymatchChar(yy, '\'')) goto l26;
  }
  l27:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "ts_quoted", yy->__buf+yy->__pos));
  return 1;
  l26:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ts_quoted", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_zero_to_255(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "zero_to_255"));
  {  int yypos30= yy->__pos, yythunkpos30= yy->__thunkpos;  if (!yymatchChar(yy, '2')) goto l31;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\077\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l31;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\077\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l31;  goto l30;
  l31:;	  yy->__pos= yypos30; yy->__thunkpos= yythunkpos30;  if (!yymatchChar(yy, '1')) goto l32;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l32;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l32;  goto l30;
  l32:;	  yy->__pos= yypos30; yy->__thunkpos= yythunkpos30;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\376\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l33;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l33;  goto l30;
  l33:;	  yy->__pos= yypos30; yy->__thunkpos= yythunkpos30;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l29;
  }
  l30:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "zero_to_255", yy->__buf+yy->__pos));
  return 1;
  l29:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "zero_to_255", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_index(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "index"));  if (!yymatchChar(yy, '[')) goto l34;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l34;
#undef yytext
#undef yyleng
  }  if (!yy_zero_to_255(yy)) goto l34;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l34;
#undef yytext
#undef yyleng
  }  if (!yymatchChar(yy, ']')) goto l34;  yyDo(yy, yy_1_index, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "index", yy->__buf+yy->__pos));
  return 1;
  l34:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "index", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_nil(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "nil"));  if (!yymatchString(yy, "NIL")) goto l35;  yyDo(yy, yy_1_nil, yy->__begin, yy->__end);  if (!yy_sp(yy)) goto l35;
  yyprintf((stderr, "  ok   %s @ %s\n", "nil", yy->__buf+yy->__pos));
  return 1;
  l35:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "nil", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_fields(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;  yyDo(yy, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "fields"));  if (!yymatchString(yy, "Fields[")) goto l36;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l36;
#undef yytext
#undef yyleng
  }
  l37:;	
  {  int yypos38= yy->__pos, yythunkpos38= yy->__thunkpos;  if (!yymatchClass(yy, (unsigned char *)"\377\377\377\377\377\377\377\377\377\377\377\337\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377")) goto l38;  goto l37;
  l38:;	  yy->__pos= yypos38; yy->__thunkpos= yythunkpos38;
  }  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l36;
#undef yytext
#undef yyleng
  }  if (!yymatchChar(yy, ']')) goto l36;  yyDo(yy, yy_1_fields, yy->__begin, yy->__end);
  {  int yypos39= yy->__pos, yythunkpos39= yy->__thunkpos;  if (!yy_index(yy)) goto l39;  yyDo(yy, yySet, -2, 0);  goto l40;
  l39:;	  yy->__pos= yypos39; yy->__thunkpos= yythunkpos39;
  }
  l40:;	  yyDo(yy, yy_2_fields, yy->__begin, yy->__end);
  {  int yypos41= yy->__pos, yythunkpos41= yy->__thunkpos;  if (!yy_index(yy)) goto l41;  yyDo(yy, yySet, -1, 0);  goto l42;
  l41:;	  yy->__pos= yypos41; yy->__thunkpos= yythunkpos41;
  }
  l42:;	  yyDo(yy, yy_3_fields, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "fields", yy->__buf+yy->__pos));  yyDo(yy, yyPop, 2, 0);
  return 1;
  l36:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "fields", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_exponent(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "exponent"));  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\000\000\040\000\000\000\040\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l43;
  {  int yypos44= yy->__pos, yythunkpos44= yy->__thunkpos;  if (!yy_sign(yy)) goto l44;  goto l45;
  l44:;	  yy->__pos= yypos44; yy->__thunkpos= yythunkpos44;
  }
  l45:;	  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l43;
  l46:;	
  {  int yypos47= yy->__pos, yythunkpos47= yy->__thunkpos;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l47;  goto l46;
  l47:;	  yy->__pos= yypos47; yy->__thunkpos= yythunkpos47;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "exponent", yy->__buf+yy->__pos));
  return 1;
  l43:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "exponent", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_decimal(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "decimal"));  if (!yymatchChar(yy, '.')) goto l48;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l48;
  l49:;	
  {  int yypos50= yy->__pos, yythunkpos50= yy->__thunkpos;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l50;  goto l49;
  l50:;	  yy->__pos= yypos50; yy->__thunkpos= yythunkpos50;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "decimal", yy->__buf+yy->__pos));
  return 1;
  l48:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "decimal", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_number(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "number"));
  {  int yypos52= yy->__pos, yythunkpos52= yy->__thunkpos;  if (!yymatchChar(yy, '0')) goto l53;  goto l52;
  l53:;	  yy->__pos= yypos52; yy->__thunkpos= yythunkpos52;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\376\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l51;
  l54:;	
  {  int yypos55= yy->__pos, yythunkpos55= yy->__thunkpos;  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l55;  goto l54;
  l55:;	  yy->__pos= yypos55; yy->__thunkpos= yythunkpos55;
  }
  }
  l52:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "number", yy->__buf+yy->__pos));
  return 1;
  l51:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "number", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_sign(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "sign"));  if (!yymatchClass(yy, (unsigned char *)"\000\000\000\000\000\050\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l56;
  yyprintf((stderr, "  ok   %s @ %s\n", "sign", yy->__buf+yy->__pos));
  return 1;
  l56:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "sign", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_numeric_value(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "numeric_value"));  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l57;
#undef yytext
#undef yyleng
  }
  {  int yypos58= yy->__pos, yythunkpos58= yy->__thunkpos;  if (!yy_sign(yy)) goto l58;  goto l59;
  l58:;	  yy->__pos= yypos58; yy->__thunkpos= yythunkpos58;
  }
  l59:;	  if (!yy_number(yy)) goto l57;
  {  int yypos60= yy->__pos, yythunkpos60= yy->__thunkpos;  if (!yy_decimal(yy)) goto l60;  goto l61;
  l60:;	  yy->__pos= yypos60; yy->__thunkpos= yythunkpos60;
  }
  l61:;	
  {  int yypos62= yy->__pos, yythunkpos62= yy->__thunkpos;  if (!yy_exponent(yy)) goto l62;  goto l63;
  l62:;	  yy->__pos= yypos62; yy->__thunkpos= yythunkpos62;
  }
  l63:;	  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l57;
#undef yytext
#undef yyleng
  }  yyDo(yy, yy_1_numeric_value, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "numeric_value", yy->__buf+yy->__pos));
  return 1;
  l57:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "numeric_value", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_numeric_headers(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "numeric_headers"));
  {  int yypos65= yy->__pos, yythunkpos65= yy->__thunkpos;  if (!yymatchString(yy, "Severity")) goto l66;  yyDo(yy, yy_1_numeric_headers, yy->__begin, yy->__end);  if (!yy_sp(yy)) goto l66;  goto l65;
  l66:;	  yy->__pos= yypos65; yy->__thunkpos= yythunkpos65;  if (!yymatchString(yy, "Pid")) goto l64;  yyDo(yy, yy_2_numeric_headers, yy->__begin, yy->__end);  if (!yy_sp(yy)) goto l64;
  }
  l65:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "numeric_headers", yy->__buf+yy->__pos));
  return 1;
  l64:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "numeric_headers", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_string_match_mod(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "string_match_mod"));  if (!yymatchChar(yy, '%')) goto l67;  yyDo(yy, yy_1_string_match_mod, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "string_match_mod", yy->__buf+yy->__pos));
  return 1;
  l67:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "string_match_mod", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_string_match(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "string_match"));
  {  int yypos69= yy->__pos, yythunkpos69= yy->__thunkpos;  if (!yy_op_seq(yy)) goto l70;  goto l69;
  l70:;	  yy->__pos= yypos69; yy->__thunkpos= yythunkpos69;  if (!yy_op_sne(yy)) goto l68;
  }
  l69:;	  if (!yy_sp(yy)) goto l68;  if (!yy_string_value(yy)) goto l68;
  {  int yypos71= yy->__pos, yythunkpos71= yy->__thunkpos;  if (!yy_string_match_mod(yy)) goto l71;  goto l72;
  l71:;	  yy->__pos= yypos71; yy->__thunkpos= yythunkpos71;
  }
  l72:;	  yyDo(yy, yy_1_string_match, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "string_match", yy->__buf+yy->__pos));
  return 1;
  l68:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "string_match", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_string_value(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "string_value"));
  {  int yypos74= yy->__pos, yythunkpos74= yy->__thunkpos;  if (!yymatchChar(yy, '"')) goto l75;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l75;
#undef yytext
#undef yyleng
  }
  l76:;	
  {  int yypos77= yy->__pos, yythunkpos77= yy->__thunkpos;
  {  int yypos78= yy->__pos, yythunkpos78= yy->__thunkpos;  if (!yymatchString(yy, "\\\"")) goto l79;  goto l78;
  l79:;	  yy->__pos= yypos78; yy->__thunkpos= yythunkpos78;
  {  int yypos80= yy->__pos, yythunkpos80= yy->__thunkpos;  if (!yymatchChar(yy, '"')) goto l80;  goto l77;
  l80:;	  yy->__pos= yypos80; yy->__thunkpos= yythunkpos80;
  }  if (!yymatchDot(yy)) goto l77;
  }
  l78:;	  goto l76;
  l77:;	  yy->__pos= yypos77; yy->__thunkpos= yythunkpos77;
  }  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l75;
#undef yytext
#undef yyleng
  }  if (!yymatchChar(yy, '"')) goto l75;  goto l74;
  l75:;	  yy->__pos= yypos74; yy->__thunkpos= yythunkpos74;  if (!yymatchChar(yy, '\'')) goto l73;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_BEGIN)) goto l73;
#undef yytext
#undef yyleng
  }
  l81:;	
  {  int yypos82= yy->__pos, yythunkpos82= yy->__thunkpos;
  {  int yypos83= yy->__pos, yythunkpos83= yy->__thunkpos;  if (!yymatchString(yy, "\\\'")) goto l84;  goto l83;
  l84:;	  yy->__pos= yypos83; yy->__thunkpos= yythunkpos83;
  {  int yypos85= yy->__pos, yythunkpos85= yy->__thunkpos;  if (!yymatchChar(yy, '\'')) goto l85;  goto l82;
  l85:;	  yy->__pos= yypos85; yy->__thunkpos= yythunkpos85;
  }  if (!yymatchDot(yy)) goto l82;
  }
  l83:;	  goto l81;
  l82:;	  yy->__pos= yypos82; yy->__thunkpos= yythunkpos82;
  }  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(YY_END)) goto l73;
#undef yytext
#undef yyleng
  }  if (!yymatchChar(yy, '\'')) goto l73;
  }
  l74:;	  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(check_string_len(yytext))) goto l73;
#undef yytext
#undef yyleng
  }  yyDo(yy, yy_1_string_value, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "string_value", yy->__buf+yy->__pos));
  return 1;
  l73:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "string_value", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_string_headers(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "string_headers"));
  {  int yypos87= yy->__pos, yythunkpos87= yy->__thunkpos;  if (!yymatchString(yy, "Type")) goto l88;  yyDo(yy, yy_1_string_headers, yy->__begin, yy->__end);  if (!yy_sp(yy)) goto l88;  goto l87;
  l88:;	  yy->__pos= yypos87; yy->__thunkpos= yythunkpos87;  if (!yymatchString(yy, "Logger")) goto l89;  yyDo(yy, yy_2_string_headers, yy->__begin, yy->__end);  if (!yy_sp(yy)) goto l89;  goto l87;
  l89:;	  yy->__pos= yypos87; yy->__thunkpos= yythunkpos87;  if (!yymatchString(yy, "Hostname")) goto l90;  yyDo(yy, yy_3_string_headers, yy->__begin, yy->__end);  if (!yy_sp(yy)) goto l90;  goto l87;
  l90:;	  yy->__pos= yypos87; yy->__thunkpos= yythunkpos87;  if (!yymatchString(yy, "EnvVersion")) goto l91;  yyDo(yy, yy_4_string_headers, yy->__begin, yy->__end);  if (!yy_sp(yy)) goto l91;  goto l87;
  l91:;	  yy->__pos= yypos87; yy->__thunkpos= yythunkpos87;  if (!yymatchString(yy, "Payload")) goto l92;  yyDo(yy, yy_5_string_headers, yy->__begin, yy->__end);  if (!yy_sp(yy)) goto l92;  goto l87;
  l92:;	  yy->__pos= yypos87; yy->__thunkpos= yythunkpos87;  if (!yymatchString(yy, "Uuid")) goto l86;  yyDo(yy, yy_6_string_headers, yy->__begin, yy->__end);  if (!yy_sp(yy)) goto l86;
  }
  l87:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "string_headers", yy->__buf+yy->__pos));
  return 1;
  l86:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "string_headers", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_boolean(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "boolean"));
  {  int yypos94= yy->__pos, yythunkpos94= yy->__thunkpos;  if (!yy_true(yy)) goto l95;  yyDo(yy, yy_1_boolean, yy->__begin, yy->__end);  goto l94;
  l95:;	  yy->__pos= yypos94; yy->__thunkpos= yythunkpos94;  if (!yy_false(yy)) goto l93;  yyDo(yy, yy_2_boolean, yy->__begin, yy->__end);
  }
  l94:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "boolean", yy->__buf+yy->__pos));
  return 1;
  l93:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "boolean", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_false(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "false"));  if (!yymatchString(yy, "FALSE")) goto l96;  if (!yy_sp(yy)) goto l96;
  yyprintf((stderr, "  ok   %s @ %s\n", "false", yy->__buf+yy->__pos));
  return 1;
  l96:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "false", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_true(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "true"));  if (!yymatchString(yy, "TRUE")) goto l97;  if (!yy_sp(yy)) goto l97;
  yyprintf((stderr, "  ok   %s @ %s\n", "true", yy->__buf+yy->__pos));
  return 1;
  l97:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "true", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_relational(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "relational"));
  {  int yypos99= yy->__pos, yythunkpos99= yy->__thunkpos;  if (!yy_op_eq(yy)) goto l100;  goto l99;
  l100:;	  yy->__pos= yypos99; yy->__thunkpos= yythunkpos99;  if (!yy_op_ne(yy)) goto l101;  goto l99;
  l101:;	  yy->__pos= yypos99; yy->__thunkpos= yythunkpos99;  if (!yy_op_gte(yy)) goto l102;  goto l99;
  l102:;	  yy->__pos= yypos99; yy->__thunkpos= yythunkpos99;  if (!yy_op_gt(yy)) goto l103;  goto l99;
  l103:;	  yy->__pos= yypos99; yy->__thunkpos= yythunkpos99;  if (!yy_op_lte(yy)) goto l104;  goto l99;
  l104:;	  yy->__pos= yypos99; yy->__thunkpos= yythunkpos99;  if (!yy_op_lt(yy)) goto l98;
  }
  l99:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "relational", yy->__buf+yy->__pos));
  return 1;
  l98:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "relational", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_op_lt(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "op_lt"));  if (!yymatchChar(yy, '<')) goto l105;  if (!yy_sp(yy)) goto l105;  yyDo(yy, yy_1_op_lt, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "op_lt", yy->__buf+yy->__pos));
  return 1;
  l105:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "op_lt", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_op_lte(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "op_lte"));  if (!yymatchString(yy, "<=")) goto l106;  if (!yy_sp(yy)) goto l106;  yyDo(yy, yy_1_op_lte, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "op_lte", yy->__buf+yy->__pos));
  return 1;
  l106:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "op_lte", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_op_gt(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "op_gt"));  if (!yymatchChar(yy, '>')) goto l107;  if (!yy_sp(yy)) goto l107;  yyDo(yy, yy_1_op_gt, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "op_gt", yy->__buf+yy->__pos));
  return 1;
  l107:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "op_gt", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_op_gte(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "op_gte"));  if (!yymatchString(yy, ">=")) goto l108;  if (!yy_sp(yy)) goto l108;  yyDo(yy, yy_1_op_gte, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "op_gte", yy->__buf+yy->__pos));
  return 1;
  l108:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "op_gte", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_op_sne(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "op_sne"));  if (!yymatchString(yy, "!~")) goto l109;  if (!yy_sp(yy)) goto l109;  yyDo(yy, yy_1_op_sne, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "op_sne", yy->__buf+yy->__pos));
  return 1;
  l109:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "op_sne", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_op_seq(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "op_seq"));  if (!yymatchString(yy, "=~")) goto l110;  if (!yy_sp(yy)) goto l110;  yyDo(yy, yy_1_op_seq, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "op_seq", yy->__buf+yy->__pos));
  return 1;
  l110:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "op_seq", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_op_ne(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "op_ne"));  if (!yymatchString(yy, "!=")) goto l111;  if (!yy_sp(yy)) goto l111;  yyDo(yy, yy_1_op_ne, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "op_ne", yy->__buf+yy->__pos));
  return 1;
  l111:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "op_ne", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_op_eq(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "op_eq"));  if (!yymatchString(yy, "==")) goto l112;  if (!yy_sp(yy)) goto l112;  yyDo(yy, yy_1_op_eq, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "op_eq", yy->__buf+yy->__pos));
  return 1;
  l112:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "op_eq", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_boolean_test(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "boolean_test"));
  {  int yypos114= yy->__pos, yythunkpos114= yy->__thunkpos;  if (!yy_true(yy)) goto l115;  yyDo(yy, yy_1_boolean_test, yy->__begin, yy->__end);  goto l114;
  l115:;	  yy->__pos= yypos114; yy->__thunkpos= yythunkpos114;  if (!yy_false(yy)) goto l113;  yyDo(yy, yy_2_boolean_test, yy->__begin, yy->__end);
  }
  l114:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "boolean_test", yy->__buf+yy->__pos));
  return 1;
  l113:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "boolean_test", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_ts_test(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "ts_test"));  if (!yymatchString(yy, "Timestamp")) goto l116;  if (!yy_sp(yy)) goto l116;  if (!yy_relational(yy)) goto l116;  if (!yy_sp(yy)) goto l116;
  {  int yypos117= yy->__pos, yythunkpos117= yy->__thunkpos;  if (!yy_numeric_value(yy)) goto l118;  goto l117;
  l118:;	  yy->__pos= yypos117; yy->__thunkpos= yythunkpos117;  if (!yy_ts_quoted(yy)) goto l116;
  }
  l117:;	  yyDo(yy, yy_1_ts_test, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ts_test", yy->__buf+yy->__pos));
  return 1;
  l116:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ts_test", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_field_test(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "field_test"));  if (!yy_fields(yy)) goto l119;  if (!yy_sp(yy)) goto l119;
  {  int yypos120= yy->__pos, yythunkpos120= yy->__thunkpos;  if (!yy_relational(yy)) goto l121;  if (!yy_sp(yy)) goto l121;
  {  int yypos122= yy->__pos, yythunkpos122= yy->__thunkpos;  if (!yy_string_value(yy)) goto l123;  goto l122;
  l123:;	  yy->__pos= yypos122; yy->__thunkpos= yythunkpos122;  if (!yy_numeric_value(yy)) goto l121;
  }
  l122:;	  goto l120;
  l121:;	  yy->__pos= yypos120; yy->__thunkpos= yythunkpos120;  if (!yy_string_match(yy)) goto l124;  goto l120;
  l124:;	  yy->__pos= yypos120; yy->__thunkpos= yythunkpos120;
  {  int yypos125= yy->__pos, yythunkpos125= yy->__thunkpos;  if (!yy_op_eq(yy)) goto l126;  goto l125;
  l126:;	  yy->__pos= yypos125; yy->__thunkpos= yythunkpos125;  if (!yy_op_ne(yy)) goto l119;
  }
  l125:;	  if (!yy_sp(yy)) goto l119;
  {  int yypos127= yy->__pos, yythunkpos127= yy->__thunkpos;  if (!yy_boolean(yy)) goto l128;  goto l127;
  l128:;	  yy->__pos= yypos127; yy->__thunkpos= yythunkpos127;  if (!yy_nil(yy)) goto l119;
  }
  l127:;	
  }
  l120:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "field_test", yy->__buf+yy->__pos));
  return 1;
  l119:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "field_test", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_numeric_test(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "numeric_test"));  if (!yy_numeric_headers(yy)) goto l129;  if (!yy_sp(yy)) goto l129;  if (!yy_relational(yy)) goto l129;  if (!yy_sp(yy)) goto l129;  if (!yy_numeric_value(yy)) goto l129;
  yyprintf((stderr, "  ok   %s @ %s\n", "numeric_test", yy->__buf+yy->__pos));
  return 1;
  l129:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "numeric_test", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_string_test(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "string_test"));  if (!yy_string_headers(yy)) goto l130;  if (!yy_sp(yy)) goto l130;
  {  int yypos131= yy->__pos, yythunkpos131= yy->__thunkpos;  if (!yy_relational(yy)) goto l132;  if (!yy_sp(yy)) goto l132;  if (!yy_string_value(yy)) goto l132;  goto l131;
  l132:;	  yy->__pos= yypos131; yy->__thunkpos= yythunkpos131;  if (!yy_string_match(yy)) goto l130;
  }
  l131:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "string_test", yy->__buf+yy->__pos));
  return 1;
  l130:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "string_test", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_close(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "close"));  if (!yymatchChar(yy, ')')) goto l133;  yyDo(yy, yy_1_close, yy->__begin, yy->__end);  if (!yy_sp(yy)) goto l133;
  yyprintf((stderr, "  ok   %s @ %s\n", "close", yy->__buf+yy->__pos));
  return 1;
  l133:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "close", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_open(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "open"));  if (!yymatchChar(yy, '(')) goto l134;  yyDo(yy, yy_1_open, yy->__begin, yy->__end);  if (!yy_sp(yy)) goto l134;
  yyprintf((stderr, "  ok   %s @ %s\n", "open", yy->__buf+yy->__pos));
  return 1;
  l134:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "open", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_test(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "test"));
  {  int yypos136= yy->__pos, yythunkpos136= yy->__thunkpos;  if (!yy_string_test(yy)) goto l137;  goto l136;
  l137:;	  yy->__pos= yypos136; yy->__thunkpos= yythunkpos136;  if (!yy_numeric_test(yy)) goto l138;  goto l136;
  l138:;	  yy->__pos= yypos136; yy->__thunkpos= yythunkpos136;  if (!yy_field_test(yy)) goto l139;  goto l136;
  l139:;	  yy->__pos= yypos136; yy->__thunkpos= yythunkpos136;  if (!yy_ts_test(yy)) goto l140;  goto l136;
  l140:;	  yy->__pos= yypos136; yy->__thunkpos= yythunkpos136;  if (!yy_boolean_test(yy)) goto l135;
  }
  l136:;	  yyDo(yy, yy_1_test, yy->__begin, yy->__end);  if (!yy_sp(yy)) goto l135;
  yyprintf((stderr, "  ok   %s @ %s\n", "test", yy->__buf+yy->__pos));
  return 1;
  l135:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "test", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_and(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "and"));  if (!yymatchString(yy, "&&")) goto l141;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(cond_cnt(&yy->ctx))) goto l141;
#undef yytext
#undef yyleng
  }  yyDo(yy, yy_1_and, yy->__begin, yy->__end);  if (!yy_sp(yy)) goto l141;
  yyprintf((stderr, "  ok   %s @ %s\n", "and", yy->__buf+yy->__pos));
  return 1;
  l141:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "and", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_expr(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "expr"));
  {  int yypos143= yy->__pos, yythunkpos143= yy->__thunkpos;  if (!yy_test(yy)) goto l144;  goto l143;
  l144:;	  yy->__pos= yypos143; yy->__thunkpos= yythunkpos143;  if (!yy_open(yy)) goto l142;  if (!yy_ored(yy)) goto l142;  if (!yy_close(yy)) goto l142;
  }
  l143:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "expr", yy->__buf+yy->__pos));
  return 1;
  l142:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "expr", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_or(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "or"));  if (!yymatchString(yy, "||")) goto l145;  yyText(yy, yy->__begin, yy->__end);  {
#define yytext yy->__text
#define yyleng yy->__textlen
if (!(cond_cnt(&yy->ctx))) goto l145;
#undef yytext
#undef yyleng
  }  yyDo(yy, yy_1_or, yy->__begin, yy->__end);  if (!yy_sp(yy)) goto l145;
  yyprintf((stderr, "  ok   %s @ %s\n", "or", yy->__buf+yy->__pos));
  return 1;
  l145:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "or", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_anded(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "anded"));  if (!yy_expr(yy)) goto l146;
  l147:;	
  {  int yypos148= yy->__pos, yythunkpos148= yy->__thunkpos;  if (!yy_and(yy)) goto l148;  if (!yy_expr(yy)) goto l148;  goto l147;
  l148:;	  yy->__pos= yypos148; yy->__thunkpos= yythunkpos148;
  }  if (!yy_sp(yy)) goto l146;
  yyprintf((stderr, "  ok   %s @ %s\n", "anded", yy->__buf+yy->__pos));
  return 1;
  l146:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "anded", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_eol(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "eol"));
  {  int yypos150= yy->__pos, yythunkpos150= yy->__thunkpos;  if (!yymatchDot(yy)) goto l150;  goto l149;
  l150:;	  yy->__pos= yypos150; yy->__thunkpos= yythunkpos150;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "eol", yy->__buf+yy->__pos));
  return 1;
  l149:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "eol", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_ored(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "ored"));  if (!yy_anded(yy)) goto l151;
  l152:;	
  {  int yypos153= yy->__pos, yythunkpos153= yy->__thunkpos;  if (!yy_or(yy)) goto l153;  if (!yy_anded(yy)) goto l153;  goto l152;
  l153:;	  yy->__pos= yypos153; yy->__thunkpos= yythunkpos153;
  }  if (!yy_sp(yy)) goto l151;
  yyprintf((stderr, "  ok   %s @ %s\n", "ored", yy->__buf+yy->__pos));
  return 1;
  l151:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ored", yy->__buf+yy->__pos));
  return 0;
}
YY_RULE(int) yy_sp(yycontext *yy)
{
  yyprintf((stderr, "%s\n", "sp"));
  l155:;	
  {  int yypos156= yy->__pos, yythunkpos156= yy->__thunkpos;  if (!yymatchClass(yy, (unsigned char *)"\000\002\000\000\001\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l156;  goto l155;
  l156:;	  yy->__pos= yypos156; yy->__thunkpos= yythunkpos156;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "sp", yy->__buf+yy->__pos));
  return 1;
}
YY_RULE(int) yy_match(yycontext *yy)
{  int yypos0= yy->__pos, yythunkpos0= yy->__thunkpos;
  yyprintf((stderr, "%s\n", "match"));  if (!yy_sp(yy)) goto l157;  if (!yy_ored(yy)) goto l157;  if (!yy_eol(yy)) goto l157;  yyDo(yy, yy_1_match, yy->__begin, yy->__end);
  yyprintf((stderr, "  ok   %s @ %s\n", "match", yy->__buf+yy->__pos));
  return 1;
  l157:;	  yy->__pos= yypos0; yy->__thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "match", yy->__buf+yy->__pos));
  return 0;
}

#ifndef YY_PART

typedef int (*yyrule)(yycontext *yy);

YY_PARSE(int) YYPARSEFROM(YY_CTX_PARAM_ yyrule yystart)
{
  int yyok;
  if (!yyctx->__buflen)
    {
      yyctx->__buflen= YY_BUFFER_SIZE;
      yyctx->__buf= (char *)YY_MALLOC(yyctx, yyctx->__buflen);
      yyctx->__textlen= YY_BUFFER_SIZE;
      yyctx->__text= (char *)YY_MALLOC(yyctx, yyctx->__textlen);
      yyctx->__thunkslen= YY_STACK_SIZE;
      yyctx->__thunks= (yythunk *)YY_MALLOC(yyctx, sizeof(yythunk) * yyctx->__thunkslen);
      yyctx->__valslen= YY_STACK_SIZE;
      yyctx->__vals= (YYSTYPE *)YY_MALLOC(yyctx, sizeof(YYSTYPE) * yyctx->__valslen);
      memset(yyctx->__vals, 0, sizeof(YYSTYPE) * yyctx->__valslen);
      yyctx->__begin= yyctx->__end= yyctx->__pos= yyctx->__limit= yyctx->__thunkpos= 0;
    }
  yyctx->__begin= yyctx->__end= yyctx->__pos;
  yyctx->__thunkpos= 0;
  yyctx->__val= yyctx->__vals;
  yyok= yystart(yyctx);
  if (yyok) yyDone(yyctx);
  yyCommit(yyctx);
  return yyok;
}

YY_PARSE(int) YYPARSE(YY_CTX_PARAM)
{
  return YYPARSEFROM(YY_CTX_ARG_ yy_match);
}

YY_PARSE(yycontext *) YYRELEASE(yycontext *yyctx)
{
  if (yyctx->__buflen)
    {
      yyctx->__buflen= 0;
      YY_FREE(yyctx, yyctx->__buf);
      YY_FREE(yyctx, yyctx->__text);
      YY_FREE(yyctx, yyctx->__thunks);
      YY_FREE(yyctx, yyctx->__vals);
    }
  return yyctx;
}

#endif
#line 382 "heka_message_matcher_parser.leg"


static match_node* copy_node(unsigned char parent, match_node *mn,
                             match_node_tmp *mnt)
{
  mn->op        = mnt->op;
  mn->val_len   = 0;
  mn->var_len   = 0;
  mn->field_id  = mnt->id;
  mn->val_mod   = mnt->val_mod;
  mn->val_type  = mnt->val_type;

  if (mn->op == OP_AND || mn->op == OP_OR) {
    mn->u.off = parent;
  } else {
    mn->u.idx.f = mnt->fi;
    mn->u.idx.a = mnt->ai;
  }

  if (mnt->id == LSB_PB_FIELDS) {
    mn->var_len = mnt->var_len;
  }
  if (mn->var_len) {
    memcpy(mn->data, mnt->var, mn->var_len); // no NUL terminator
    free(mnt->var);
    mnt->var = NULL;
    mnt->var_len = 0;
  }

  size_t val_len = 0;
  switch (mnt->val_type) {
  case TYPE_STRING:
    val_len = mnt->val_len + 1;
    memcpy(mn->data + mn->var_len, mnt->val.s, val_len);
    free(mnt->val.s);
    mnt->val.s = NULL;
    mnt->val_len = 0;
    break;
  case TYPE_NUMERIC:
    val_len = sizeof(double);
    memcpy(mn->data + mn->var_len, &mnt->val.d, val_len);
    break;
  default:
    break;
  }

  mn->units = 1 + ((sizeof(match_node) - 1 + mn->var_len + val_len)
                   / sizeof(match_node));
  if (val_len && mnt->val_type == TYPE_STRING) {
    mn->val_len = val_len - 1;
  } else {
    mn->val_len = val_len;
  }
  return mn + mn->units;
}


static match_node* inorder_traverse(unsigned char parent, match_node **offsets,
                                    match_node *mn, match_node_tmp *root,
                                    match_node_tmp *mnt)
{
  if (mnt->id == 0 && (mnt->op == OP_AND || mnt->op == OP_OR)) {
    unsigned char idx = mnt->val_len;
    mnt->val_len = 0;
    mn = inorder_traverse(idx, offsets, mn, root, root + mnt->fi);
    offsets[idx] = mn;
    mn = copy_node(parent, mn, mnt);
  } else {
    mn = copy_node(parent, mn, mnt);
  }
  if (mnt->id == 0 && (mnt->op == OP_AND || mnt->op == OP_OR)) {
    mn = inorder_traverse(parent, offsets, mn, root, root + mnt->ai);
  }
  return mn;
}


static size_t get_matcher_bytes(match_node_tmp nodes[], size_t size)
{
  size_t len = 0;
  for (unsigned i = 0; i < size; ++i) {
    size_t val_len = 0;
    switch (nodes[i].val_type) {
    case TYPE_STRING:
      val_len = nodes[i].val_len + 1;
      break;
    case TYPE_NUMERIC:
      val_len = sizeof(double);
      break;
    default:
      break;
    }

    size_t var_len = 0;
    if (nodes[i].id == LSB_PB_FIELDS) {
      var_len = nodes[i].var_len;
    }

    len += (sizeof(match_node) * 2 + val_len + var_len - 1)
      / sizeof(match_node) * sizeof(match_node);

    if (nodes[i].op == OP_OR || nodes[i].op == OP_AND) {
      // squirrel away the position for the short-circuit calculation
      nodes[i].val_len = i;
    }
  }
  return len;
}


static void make_tree(match_node_tmp nodes[], size_t size)
{
  // turn the postfix stack into a traversable tree
  match_node_tmp *stack[size];
  memset(stack, 0, sizeof(stack));
  int top = 0;
  for (unsigned i = 0; i < size; ++i) {
    if (nodes[i].op != OP_AND && nodes[i].op != OP_OR) {
      stack[top++] = &nodes[i];
    } else {
      nodes[i].ai = stack[--top] - nodes;
      nodes[i].fi = stack[--top] - nodes;
      stack[top++] = &nodes[i];
    }
  }
}


static lsb_message_matcher* make_matcher(match_node_tmp nodes[], size_t size)
{
  lsb_message_matcher *mm = malloc(sizeof(lsb_message_matcher));
  if (!mm) { return NULL; }

  mm->bytes = get_matcher_bytes(nodes, size);
  mm->nodes = calloc(mm->bytes, 1);
  if (!mm->nodes) {
    free(mm);
    return NULL;
  }

  match_node *offsets[size];
  memset(offsets, 0, sizeof(offsets));

  inorder_traverse(size, offsets, mm->nodes, nodes, nodes + (size - 1));

  // populate the short-circuit offsets
  match_node *e = mm->nodes + (mm->bytes / sizeof(match_node));
  for (match_node *p = mm->nodes; p < e;){
    if (p->op == OP_AND || p->op == OP_OR) {
      if (p->u.off < size) {
        p->u.off = offsets[p->u.off] - mm->nodes;
      } else {
        p->u.off =  mm->bytes / sizeof(match_node);
      }
    }
    p += p->units;
  }
  return mm;
}


lsb_message_matcher* lsb_create_message_matcher(const char *exp)
{
  if (!exp) { return NULL; }

  lsb_message_matcher *mm = NULL;
  yycontext yy;
  memset(&yy, 0, sizeof(yy));
  yy.ctx.is.s = exp;
  yy.ctx.is.size = strlen(exp);
  int ret = yyparse(&yy);
  if (ret) {
    make_tree(yy.ctx.out.a, yy.ctx.out.pos);
    mm = make_matcher(yy.ctx.out.a, yy.ctx.out.pos);
  }
  free(yy.ctx.out.a);
  free(yy.ctx.ops.a);
  yyrelease(&yy);
  return mm;
}

